function [f_edg] = cost_2nodes(nodeX,nodeY,nodeTar,theta_polar,v_polar_wind,v_polar_propeller,polar_sun,wind_angl,sun_azi,sun_elv)               
% cost function between two nodes
                %--------------------------------
                xy_node = nodeX.pos;
                xy_adj = nodeY.pos;
                xy_tar = nodeTar.pos;
                
                %---------------------------------
                L_edg = sqrt((xy_node(1)-xy_adj(1)).^2+(xy_node(2)-xy_adj(2)).^2);
                theta_edg = atan2(xy_adj(1)-xy_node(1),xy_adj(2)-xy_node(2));
                
                % interpolate vboat, v_seaCurrent, solar output
                % calibrate the angle
                if (theta_edg<0*pi/180)
                    theta_edg = - theta_edg;
                end
                
                theta_edg_N = theta_edg;
                
                theta_relative = abs(wind_angl-theta_edg);% relative angle between wind direction and boat heading
                
                % sailboat: interpolate between different curves
                if (theta_relative>45*pi/180)
                    v_boat = interp1(theta_polar,v_polar_wind,theta_relative);
                else
                    v_boat = interp1(theta_polar,v_polar_propeller,theta_relative);
                end
                % sea current map
                [x,y] = meshgrid(0:5:50,0:5:50);
                xx_sea = cos(x/25).*(y/25);
                yy_sea = sin(x/25).*(y/25);
                xq = (xy_node(1) + xy_adj(1))/2;
                yq = (xy_node(2) + xy_adj(2))/2;
                sc_x = interp2(x,y,xx_sea,xq,yq);
                sc_y = interp2(x,y,yy_sea,xq,yq);
                v_sea = sqrt(sc_x.^2 + sc_y.^2);
                theta_sea = atan2(sc_x,sc_y);
                
                % solar diagram
                
                 
                
                % unit vector of sun's ray: from azimuth and elevation of the
                % sun
                n_sr_x = cos(sun_elv)*cos(sun_azi);
                n_sr_y = cos(sun_elv)*sin(sun_azi);
                n_sr_z = sin(sun_elv);
                n_sunray = [n_sr_x n_sr_y n_sr_z]';
                
                % normal vector of the solar-sail plane: from yaw angle (psi) of
                % the sail, it is a function of the roll angle (phi)
                
                % rotation matrix from I-frame to B-frame: roll = 0, pitch
                % = 0, yaw = theta_edg_N
                R = [cos(theta_edg_N) sin(theta_edg_N);-sin(theta_edg_N) cos(theta_edg_N)];
                n_sr_x_B = R(1,1)*n_sr_x + R(1,2)*n_sr_y;
                n_sr_y_B = R(2,1)*n_sr_x + R(2,2)*n_sr_y;
                n_sr_z_B = n_sr_z;
                
                
                % approximate these two angles to find the optimal roll
                % angle, and the angle between sun'ray and solar-sail plane
                psi_s_B = (pi/2)*(cos(theta_edg)+1)/2;% from theta_edg! this is computed by maximizing the force along x_B?
                
                phi_s_B = real(acos(n_sr_x_B/cos(psi_s_B)));% have to compute?
                
                %n_p_x = cos(phi_s_B)*cos(psi_s_B); % n_p_x = n_sr_x -> phi_B angle
                %n_p_y = cos(phi_s_B)*sin(psi_s_B); % n_p_y = n_sr_y -> 
                %n_p_z = sin(phi_s_B); % n_p_z = n_sr_z -> 
                %n_plane = [n_p_x n_p_y n_p_z]';
                
                
                
                % check constraints: 
                 if phi_s_B >= 45*pi/180,
                     phi_s_B = 45*pi/180;
                 end
                
                
                % interpolate the power from the polar plot of the sun
                
                phi_sun = 0:15*pi/180:pi;% angle between solar panel and sun's rays
                P_sun = interp1(phi_sun,polar_sun,phi_s_B);
                %----------------
                % g-cost function: energy consumption and energy production
                wc = 1;
                wp = 1;
                % energy consumption
                if (theta_relative<=45*pi/180)
                    Ec = ((v_boat-v_sea*cos(theta_edg-theta_sea))^2)*L_edg/v_boat;
                else
                    Ec = 0;
                end
                % energy production
                w_s = 1;
                w_hyo = 1;
                E_sun = P_sun*L_edg/v_boat;
                if (theta_relative<=45*pi/180),
                    E_hyo = 0;
                else
                    E_hyo = ((v_boat-v_sea*cos(theta_edg-theta_sea))^2)*L_edg/v_boat;
                end
                Ep = w_s*E_sun + w_hyo*E_hyo;
                %gg = ((v_boat-v_sea*cos(theta_edg-theta_sea)).^3/v_boat)*L_edg;
                gg = wc*Ec - wp*Ep;                
                %---------------------
                % h-heuristic function: distance from current point to target point
                hh = sqrt((xy_tar(1)-xy_adj(1)).^2+(xy_tar(2)-xy_adj(2)).^2);
                
                % Total function: f-function f = wg*gg + wh*hh
                wg = 1;
                wh = 1;
                f_edg = wg*gg + wh*hh;
end